@@ -91,6 +91,8 @@ struct dav_node {
        changed remotely. If present it overrides information from the
        Last-Modified time (smtime). */
     char *etag;
+    /* The media-type as in HTTP-header Content-Type. */
+    char *mime_type;
     /* A linked list of handles for open files. */
     dav_handle *handles;
     /* Size of the contents of the node.
@@ -136,16 +138,25 @@ struct dav_node {
 };
 
 
+/* An item in a linked list. Each item holds a pointer to a dav_node.*/
+typedef struct dav_node_item dav_node_list_item;
+struct dav_node_item {
+    dav_node_list_item *next;
+    dav_node *node;
+    time_t save_at;
+};
+
+
 /* Returned by dav_statfs(). */
 typedef struct dav_stat dav_stat;
 struct dav_stat {
-    unsigned long long blocks;
-    unsigned long long bfree;
-    unsigned long long bavail;
-    unsigned long long files;
-    unsigned long long ffree;
-    unsigned long long bsize;
-    unsigned long long namelen;
+    off_t blocks;
+    off_t bfree;
+    off_t bavail;
+    off_t files;
+    off_t ffree;
+    off_t bsize;
+    off_t namelen;
 };
 
 
@@ -179,10 +190,11 @@ typedef off_t (*dav_write_dir_entry_fn)(int fd, off_t off, const dav_node *node,
    paramters: if not self explaining, please see mount_davfs.c, struct args. */
 void dav_init_cache(uid_t uid, gid_t gid, mode_t file_mode, mode_t dir_mode,
                     mode_t f_umask, mode_t d_umask, size_t hash_table_sz,
-                    time_t dir_rf, time_t file_rf, int gui_opt, time_t dir_rt,
-                    time_t dir_max_rt, time_t lock_rf, size_t cache_size,
-                    const char *tl_cache_dir, const char *backup_dir,
-                    const char *host, const char *path, const char *mpoint);
+                    time_t dir_rf, time_t file_rf, int delay_upl, int gui_opt,
+                    time_t dir_rt, time_t dir_max_rt, time_t lock_rf,
+                    size_t cache_sz, const char *tl_cache_dir,
+                    const char *backup_dir, const char *host, const char *path,
+                    const char *mpoint, int dbg);
 
 
 /* Tries to write back to the server all open files that have been changed or
@@ -190,8 +202,9 @@ void dav_init_cache(uid_t uid, gid_t gid, mode_t file_mode, mode_t dir_mode,
    a local backup file is created. All local copies of files and the necessary
    directories are stored in the permanent cache. A new index file of the
    permanent cache is created.
+   If got_sigterm is 1, dirty files will not be stored back to the server.
    Finally it frees all nodes. */
-void dav_close_cache();
+void dav_close_cache(int got_sigterm);
 
 
 /* Registers the kernel_interface.
@@ -210,7 +223,7 @@ size_t dav_register_kernel_interface(dav_write_dir_entry_fn write_fn,
    The return value indicates whether another run would be useful.
    return value: 0 there is nothing left to do.
                  1 another call of dav_tidy_cache would be useful. */
-int dav_tidy_cache();
+int dav_tidy_cache(void);
 
 
 /* Upcalls from the kernel, via the interface module. */
@@ -277,6 +290,16 @@ int dav_create(dav_node **nodep, dav_node *parent, const char *name,
 int dav_getattr(dav_node *node, uid_t uid);
 
 
+/* Checks whether node exists and uid has permissions. The value of
+   extended attribute name is copied into buf. If its size is greater
+   than size, EOVERFLOW is returned.
+   Permissions:
+   uid must have execute permission for parent and all of its ancestors, as
+   well as read permission for parent. */
+int dav_getxattr(dav_node *node, const char *name, char *buf, size_t *size,
+                uid_t uid);
+
+
 /* Searches for a node with name name in the directory parent and returns the
    node in nodep.
    Permissions:
@@ -367,7 +390,6 @@ int dav_setattr(dav_node *node, uid_t uid, int sm, mode_t mode,
                 int so, uid_t owner, int sg, gid_t gid, int sat, time_t atime,
                 int smt, time_t mtime, int ssz, off_t size);
 
-
 /* Returns struct dav_stat which currently is mainly fake.
    No permissions necessary. */
 dav_stat dav_statfs(void);
