@@ -44,7 +44,6 @@
 #   define _(String) String
 #endif
 
-#include "dav_debug.h"
 #include "defaults.h"
 #include "cache.h"
 #include "kernel_interface.h"
@@ -56,7 +55,7 @@
 
 void dav_coda2_loop(int device, size_t bufsize, time_t idle_time,
                     dav_is_mounted_fn is_mounted,
-                    volatile int *keep_on_running) {
+                    volatile int *keep_on_running, int dbg) {
     kill(getppid(), SIGHUP);
     return;
 }
@@ -91,6 +90,9 @@ static unsigned int blocksize;
    Used to compute file numbers from node pointers. */
 static size_t alignment;
 
+/* Send debug messages to syslog if != 0. */
+int debug;
+
 
 /* Private function prototypes */
 /*=============================*/
@@ -134,7 +136,11 @@ static void set_attr(struct coda_vattr *attr, const dav_node *node);
 
 void dav_coda2_loop(int device, size_t bufsize, time_t idle_time,
                     dav_is_mounted_fn is_mounted,
-                    volatile int *keep_on_running) {
+                    volatile int *keep_on_running, int dbg) {
+
+    debug = dbg;
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "coda kernel version 2");
 
     buf = malloc(BUF_SIZE);
     if (buf == NULL) {
@@ -150,7 +156,6 @@ void dav_coda2_loop(int device, size_t bufsize, time_t idle_time,
     tv.tv_sec = idle_time;
     tv.tv_usec = 0;
     time_t last_tidy_cache = time(NULL);
-    int delay_tidy = 0;
 
     while (*keep_on_running) {
 
@@ -191,139 +196,153 @@ void dav_coda2_loop(int device, size_t bufsize, time_t idle_time,
         uint32_t len;
         switch (ih->opcode) {
         case CODA_ROOT:
-            DBG0("CODA_ROOT:");
             len = coda_root();
             break;
         case CODA_OPEN_BY_FD:
-            DBG0("CODA_OPEN_BY_FD:");
             len = coda_open_by_fd();
             break;
         case CODA_OPEN:
-            DBG0("CODA_OPEN: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_OPEN:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_CLOSE:
-            DBG0("CODA_CLOSE:");
             len = coda_close();
-            delay_tidy = 1;
+            last_tidy_cache = 0;
             break;
         case CODA_IOCTL:
-            DBG0("CODA_IOCTL: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_IOCTL:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_GETATTR:
-            DBG0("CODA_GETATTR:");
             len = coda_getattr();
             break;
         case CODA_SETATTR:
-            DBG0("CODA_SETATTR:");
             len = coda_setattr();
             break;
         case CODA_ACCESS:
-            DBG0("CODA_ACCESS:");
             len = coda_access();
             break;
         case CODA_LOOKUP:
-            DBG0("CODA_LOOKUP:");
             len = coda_lookup();
             break;
         case CODA_CREATE:
-            DBG0("CODA_CREATE:");
             len = coda_create();
             break;
         case CODA_REMOVE: {
-            DBG0("CODA_REMOVE:");
             struct coda_remove_in *in = (struct coda_remove_in *) buf;
-            DBG2("  p 0x%x, %s", in->VFid.opaque[2], buf + in->name);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_REMOVE:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%x, %s",
+                       in->VFid.opaque[2], buf + in->name);
+            }
             oh->result = dav_remove((dav_node *) in->VFid.opaque[2],
                                     buf + in->name, ih->cred.cr_euid);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_LINK:
-            DBG0("CODA_LINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_LINK:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_RENAME: {
-            DBG0("CODA_RENAME:");
             struct coda_rename_in *in = (struct coda_rename_in *) buf;
-            DBG2("  sp 0x%x, %s", in->sourceFid.opaque[2], buf + in->srcname);
-            DBG2("  dp 0x%x, %s", in->destFid.opaque[2], buf + in->destname);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_RENAME:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  sp 0x%x, %s",
+                       in->sourceFid.opaque[2], buf + in->srcname);
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  dp 0x%x, %s",
+                       in->destFid.opaque[2], buf + in->destname);
+            }
             oh->result = dav_rename((dav_node *) in->sourceFid.opaque[2],
                                     buf + in->srcname,
                                     (dav_node *) in->destFid.opaque[2],
                                     buf + in->destname, ih->cred.cr_euid);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_MKDIR:
-            DBG0("CODA_MKDIR:");
             len = coda_mkdir();
             break;
         case CODA_RMDIR: {
-            DBG0("CODA_RMDIR:");
             struct coda_rmdir_in *in = (struct coda_rmdir_in *) buf;
-            DBG2("  p 0x%x, %s", in->VFid.opaque[2], buf + in->name);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_RMDIR:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%x, %s",
+                       in->VFid.opaque[2], buf + in->name);
+            }
             oh->result = dav_rmdir((dav_node *) in->VFid.opaque[2],
                                    buf + in->name, ih->cred.cr_euid);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_SYMLINK:
-            DBG0("CODA_SYMLINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_SYMLINK:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_READLINK:
-            DBG0("CODA_READLINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_READLINK:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_FSYNC: {
-            DBG0("CODA_FSYNC:");
             struct coda_fsync_in *in = (struct coda_fsync_in *) buf;
-            DBG1("  n 0x%x", in->VFid.opaque[2]);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_FSYNC:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%x",
+                       in->VFid.opaque[2]);
+            }
             oh->result = dav_sync((dav_node *) in->VFid.opaque[2]);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_VGET:
-            DBG0("CODA_VGET: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_VGET:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_OPEN_BY_PATH:
-            DBG0("CODA_OPEN_BY_PATH: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "CODA_OPEN_BY_PATH:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_STATFS:
-            DBG0("CODA_STATFS:");
             len = coda_statfs();
             break;
         case CODA_STORE: {
-            DBG0("CODA_STORE:");
             struct coda_store_in *in = (struct coda_store_in *) buf;
-            DBG2("  n 0x%x, f 0x%x", in->VFid.opaque[2], in->flags);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_STORE:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%x, f 0x%x",
+                       in->VFid.opaque[2], in->flags);
+            }
             oh->result = dav_sync((dav_node *) in->VFid.opaque[2]);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_RELEASE:
-            DBG0("CODA_RELEASE:");
             len = coda_close();
-            delay_tidy = 1;
+            last_tidy_cache = 0;
             break;
         default:
-            DBG1("UNKNOWN CODA CALL %u", ih->opcode);
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "UNKNOWN CODA CALL %u", ih->opcode);
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         }
 
+        if (debug)
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "RET: %s",
+                   strerror(oh->result));
+
         ssize_t n = 0;
         ssize_t w = 0;
         while (n < len && w >= 0) {
@@ -333,15 +352,10 @@ void dav_coda2_loop(int device, size_t bufsize, time_t idle_time,
 
         if (time(NULL) < (last_tidy_cache + idle_time)) {
             tv.tv_sec = last_tidy_cache + idle_time - time(NULL);
-        } else if (delay_tidy) {
-            tv.tv_sec = 1;
-            last_tidy_cache = time(NULL) - idle_time + 2;
-            delay_tidy = 0;
-        } else if (dav_tidy_cache() == 0) {
+        } else {
+            dav_tidy_cache();
             tv.tv_sec = idle_time;
             last_tidy_cache = time(NULL);
-        } else {
-            tv.tv_sec = 0;
         }
 
         if (flush) {
@@ -368,7 +382,11 @@ static uint32_t coda_access(void) {
     struct coda_in_hdr *ih = (struct coda_in_hdr *) buf;
     struct coda_access_in *in = (struct coda_access_in *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
-    DBG2("  n 0x%x, f %x", in->VFid.opaque[2], in->flags);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_ACCESS:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%x, f %x",
+               in->VFid.opaque[2], in->flags);
+    }
 
     int how = (in->flags & C_A_R_OK) ? R_OK : 0;
     how |= (in->flags & C_A_W_OK) ? W_OK : 0;
@@ -378,7 +396,6 @@ static uint32_t coda_access(void) {
     oh->result = dav_access((dav_node *) in->VFid.opaque[2], ih->cred.cr_euid,
                             how);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_out_hdr);
 }
 
@@ -388,8 +405,17 @@ static uint32_t coda_close(void) {
     struct coda_in_hdr *ih = (struct coda_in_hdr *) buf;
     struct coda_close_in *in = (struct coda_close_in *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
-    DBG2("  n 0x%x, f %x", in->VFid.opaque[2], in->flags);
-    DBG2("  pid %i, pgid %i", ih->pid, ih->pgid);
+    if (debug) {
+        if (ih->opcode == CODA_CLOSE) {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_CLOSE:");
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_RELEASE:");
+        }
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%x, f %x",
+               in->VFid.opaque[2], in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, pgid %i",
+               ih->pid, ih->pgid);
+    }
 
     int flags = 0;
     if ((in->flags & C_O_READ) && (in->flags & C_O_WRITE)) {
@@ -403,7 +429,6 @@ static uint32_t coda_close(void) {
     oh->result = dav_close((dav_node *) in->VFid.opaque[2], 0, flags, ih->pid,
                            ih->pgid);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_out_hdr);
 }
 
@@ -414,8 +439,12 @@ static uint32_t coda_create(void) {
     struct coda_create_in *in = (struct coda_create_in *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_create_out *out = (struct coda_create_out *) buf;
-    DBG2("  p 0x%x, m %o", in->VFid.opaque[2], in->mode);
-    DBG1("  %s", buf + in->name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_CREATE:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%x, m %o",
+               in->VFid.opaque[2], in->mode);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  %s", buf + in->name);
+    }
 
     dav_node *node = NULL;
     oh->result = dav_create(&node, (dav_node *) in->VFid.opaque[2],
@@ -425,7 +454,6 @@ static uint32_t coda_create(void) {
     if (oh->result != 0 || node == NULL) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
@@ -434,7 +462,6 @@ static uint32_t coda_create(void) {
     out->VFid.opaque[2] = (size_t) node;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_create_out);
 }
 
@@ -445,12 +472,15 @@ static uint32_t coda_getattr(void) {
     struct coda_getattr_in *in = (struct coda_getattr_in *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_getattr_out *out = (struct coda_getattr_out *) buf;
-    DBG1("  n 0x%x", in->VFid.opaque[2]);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_GETATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%x",
+               in->VFid.opaque[2]);
+    }
 
     oh->result = dav_getattr((dav_node *) in->VFid.opaque[2],
                              ih->cred.cr_euid);
 
-    DBG_RET(oh->result);
     if (oh->result != 0)
         return sizeof(struct coda_out_hdr);
 
@@ -466,7 +496,11 @@ static uint32_t coda_lookup(void) {
     struct coda_lookup_in *in = (struct coda_lookup_in *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_lookup_out *out = (struct coda_lookup_out *) buf;
-    DBG2("  p 0x%x, %s", in->VFid.opaque[2], buf + in->name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_LOOKUP:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%x, %s",
+               in->VFid.opaque[2], buf + in->name);
+    }
 
     dav_node *node = NULL;
     oh->result = dav_lookup(&node, (dav_node *) in->VFid.opaque[2],
@@ -475,7 +509,6 @@ static uint32_t coda_lookup(void) {
     if (oh->result != 0 || node == NULL) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
@@ -484,7 +517,6 @@ static uint32_t coda_lookup(void) {
     out->VFid.opaque[2] = (size_t) node;
     out->vtype = (node->mode & S_IFDIR) ? CDT_DIR : CDT_REG;
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_lookup_out);
 }
 
@@ -495,7 +527,11 @@ static uint32_t coda_mkdir(void) {
     struct coda_mkdir_in *in = (struct coda_mkdir_in *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_mkdir_out *out = (struct coda_mkdir_out *) buf;
-    DBG2("  p 0x%x, %s", in->VFid.opaque[2], buf + in->name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_MKDIR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%x, %s",
+               in->VFid.opaque[2], buf + in->name);
+    }
 
     dav_node *node = NULL;
     oh->result = dav_mkdir(&node, (dav_node *) in->VFid.opaque[2],
@@ -505,7 +541,6 @@ static uint32_t coda_mkdir(void) {
     if (oh->result != 0 || node == NULL) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
@@ -514,7 +549,6 @@ static uint32_t coda_mkdir(void) {
     out->VFid.opaque[2] = (size_t) node;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_mkdir_out);
 }
 
@@ -525,8 +559,13 @@ static uint32_t coda_open_by_fd(void) {
     struct coda_open_by_fd_in *in = (struct coda_open_by_fd_in *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_open_by_fd_out *out = (struct coda_open_by_fd_out *) buf;
-    DBG2("  n 0x%x, f %x", in->VFid.opaque[2], in->flags);
-    DBG2("  pid %i, pgid %i", ih->pid, ih->pgid);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_OPEN_BY_FD:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%x, f %x",
+               in->VFid.opaque[2], in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, pgid %i",
+               ih->pid, ih->pgid);
+    }
 
     int flags = 0;
     if ((in->flags & C_O_READ) && (in->flags & C_O_WRITE)) {
@@ -544,11 +583,9 @@ static uint32_t coda_open_by_fd(void) {
     if (oh->result != 0 || out->fd == 0) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_open_by_fd_out);
 }
 
@@ -558,6 +595,8 @@ static uint32_t coda_root(void) {
     struct coda_in_hdr *ih = (struct coda_in_hdr *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_root_out *out = (struct coda_root_out *) buf;
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_ROOT:");
 
     dav_node *node = NULL;
     oh->result = dav_root(&node, ih->cred.cr_euid);
@@ -565,7 +604,6 @@ static uint32_t coda_root(void) {
     if (oh->result != 0 || node == NULL) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
@@ -573,7 +611,6 @@ static uint32_t coda_root(void) {
     out->VFid.opaque[1] = DAV_VNODE;
     out->VFid.opaque[2] = (size_t) node;
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_root_out);
 }
 
@@ -583,11 +620,17 @@ static uint32_t coda_setattr(void) {
     struct coda_in_hdr *ih = (struct coda_in_hdr *) buf;
     struct coda_setattr_in *in = (struct coda_setattr_in *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
-    DBG2("  n 0x%x, m %o", in->VFid.opaque[2], in->attr.va_mode);
-    DBG2("  uid: %i, gid: %i", in->attr.va_uid, in->attr.va_gid);
-    DBG2("  at %li, mt %li", in->attr.va_atime.tv_sec,
-         in->attr.va_mtime.tv_sec);
-    DBG2("  ct %li, sz %llu", in->attr.va_ctime.tv_sec, in->attr.va_size);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_SETATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%x, m %o",
+               in->VFid.opaque[2], in->attr.va_mode);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  uid: %i, gid: %i",
+               in->attr.va_uid, in->attr.va_gid);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  at %li, mt %li",
+               in->attr.va_atime.tv_sec, in->attr.va_mtime.tv_sec);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  ct %li, sz %llu",
+               in->attr.va_ctime.tv_sec, in->attr.va_size);
+    }
 
     oh->result = dav_setattr((dav_node *) in->VFid.opaque[2], ih->cred.cr_euid,
                              in->attr.va_mode != USHRT_MAX,
@@ -601,7 +644,6 @@ static uint32_t coda_setattr(void) {
                              in->attr.va_size != UINT64_MAX,
                              in->attr.va_size);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_out_hdr);
 }
 
@@ -610,6 +652,8 @@ static uint32_t coda_statfs(void) {
 
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_statfs_out *out = (struct coda_statfs_out *) buf;
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_STATFS:");
 
     dav_stat st = dav_statfs();
 
@@ -620,7 +664,6 @@ static uint32_t coda_statfs(void) {
     out->stat.f_ffree = st.ffree;
 
     oh->result = 0;
-    DBG_RET(oh->result);
     return sizeof(struct coda_statfs_out);
 }
 
@@ -631,7 +674,8 @@ static uint32_t coda_statfs(void) {
 static void coda_flush(int device) {
 
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
-    DBG0("  CODA_FLUSH:");
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  CODA_FLUSH:");
 
     oh->opcode = CODA_FLUSH;
     oh->unique = 0;
