@@ -43,7 +43,6 @@
 #   define _(String) String
 #endif
 
-#include "dav_debug.h"
 #include "defaults.h"
 #include "cache.h"
 #include "kernel_interface.h"
@@ -76,6 +75,9 @@ static unsigned int blocksize;
    Used to compute file numbers from node pointers. */
 static size_t alignment;
 
+/* Send debug messages to syslog if != 0. */
+int debug;
+
 
 /* Private function prototypes */
 /*=============================*/
@@ -119,7 +121,11 @@ static void set_attr(struct coda_vattr *attr, const dav_node *node);
 
 void dav_coda3_loop(int device, size_t bufsize, time_t idle_time,
                     dav_is_mounted_fn is_mounted,
-                    volatile int *keep_on_running) {
+                    volatile int *keep_on_running, int dbg) {
+
+    debug = dbg;
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "coda kernel version 3");
 
     buf = malloc(BUF_SIZE);
     if (buf == NULL) {
@@ -135,7 +141,6 @@ void dav_coda3_loop(int device, size_t bufsize, time_t idle_time,
     tv.tv_sec = idle_time;
     tv.tv_usec = 0;
     time_t last_tidy_cache = time(NULL);
-    int delay_tidy = 0;
 
     while (*keep_on_running) {
 
@@ -176,141 +181,154 @@ void dav_coda3_loop(int device, size_t bufsize, time_t idle_time,
         uint32_t len;
         switch (ih->opcode) {
         case CODA_ROOT:
-            DBG0("CODA_ROOT:");
             len = coda_root();
             break;
         case CODA_OPEN_BY_FD:
-            DBG0("CODA_OPEN_BY_FD:");
             len = coda_open_by_fd();
             break;
         case CODA_OPEN:
-            DBG0("CODA_OPEN: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_OPEN:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_CLOSE:
-            DBG0("CODA_CLOSE:");
             len = coda_close();
-            delay_tidy = 1;
+            last_tidy_cache = 0;
             break;
         case CODA_IOCTL:
-            DBG0("CODA_IOCTL: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_IOCTL:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_GETATTR:
-            DBG0("CODA_GETATTR:");
             len = coda_getattr();
             break;
         case CODA_SETATTR:
-            DBG0("CODA_SETATTR:");
             len = coda_setattr();
             break;
         case CODA_ACCESS:
-            DBG0("CODA_ACCESS:");
             len = coda_access();
             break;
         case CODA_LOOKUP:
-            DBG0("CODA_LOOKUP:");
             len = coda_lookup();
             break;
         case CODA_CREATE:
-            DBG0("CODA_CREATE:");
             len = coda_create();
             break;
         case CODA_REMOVE: {
-            DBG0("CODA_REMOVE:");
             struct coda_remove_in *in = (struct coda_remove_in *) buf;
             dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
-            DBG2("  p %p, %s", node, buf + in->name);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_REMOVE:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p %p, %s", node,
+                       buf + in->name);
+            }
             oh->result = dav_remove(node, buf + in->name, ih->uid);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_LINK:
-            DBG0("CODA_LINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_LINK:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_RENAME: {
-            DBG0("CODA_RENAME:");
             struct coda_rename_in *in = (struct coda_rename_in *) buf;
             dav_node *src = *((dav_node **) &(in->sourceFid.opaque[2]));
             dav_node *dst = *((dav_node **) &(in->destFid.opaque[2]));
-            DBG2("  sp %p, %s", src, buf + in->srcname);
-            DBG2("  dp %p, %s", dst, buf + in->destname);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_RENAME:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  sp %p, %s", src,
+                       buf + in->srcname);
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  dp %p, %s", dst,
+                       buf + in->destname);
+            }
             oh->result = dav_rename(src, buf + in->srcname, dst,
                                     buf + in->destname, ih->uid);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_MKDIR:
-            DBG0("CODA_MKDIR:");
             len = coda_mkdir();
             break;
         case CODA_RMDIR: {
-            DBG0("CODA_RMDIR:");
             struct coda_rmdir_in *in = (struct coda_rmdir_in *) buf;
             dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
-            DBG2("  p %p, %s", node, buf + in->name);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_RMDIR:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p %p, %s", node,
+                       buf + in->name);
+            }
             oh->result = dav_rmdir(node, buf + in->name, ih->uid);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_SYMLINK:
-            DBG0("CODA_SYMLINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_SYMLINK:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_READLINK:
-            DBG0("CODA_READLINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_READLINK:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_FSYNC: {
-            DBG0("CODA_FSYNC:");
             struct coda_fsync_in *in = (struct coda_fsync_in *) buf;
             dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
-            DBG1("  n %p", node);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_FSYNC:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n %p", node);
+            }
             oh->result = dav_sync(node);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_VGET:
-            DBG0("CODA_VGET: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_VGET:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_OPEN_BY_PATH:
-            DBG0("CODA_OPEN_BY_PATH: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "CODA_OPEN_BY_PATH:");
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         case CODA_STATFS:
-            DBG0("CODA_STATFS:");
             len = coda_statfs();
             break;
         case CODA_STORE: {
-            DBG0("CODA_STORE:");
             struct coda_store_in *in = (struct coda_store_in *) buf;
             dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
-            DBG2("  n %p, f 0x%x", node, in->flags);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_STORE:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n %p, f 0x%x",
+                       node, in->flags);
+            }
             oh->result = dav_sync(node);
-            DBG_RET(oh->result);
             len = sizeof(struct coda_out_hdr);
             break; }
         case CODA_RELEASE:
-            DBG0("CODA_RELEASE:");
             len = coda_close();
-            delay_tidy = 1;
+            last_tidy_cache = 0;
             break;
         default:
-            DBG1("UNKNOWN CODA CALL %u", ih->opcode);
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "UNKNOWN CODA CALL %u", ih->opcode);
             oh->result = ENOTSUP;
             len = sizeof(struct coda_out_hdr);
             break;
         }
 
+        if (debug)
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "RET: %s",
+                   strerror(oh->result));
+
         ssize_t n = 0;
         ssize_t w = 0;
         while (n < len && w >= 0) {
@@ -320,15 +338,10 @@ void dav_coda3_loop(int device, size_t bufsize, time_t idle_time,
 
         if (time(NULL) < (last_tidy_cache + idle_time)) {
             tv.tv_sec = last_tidy_cache + idle_time - time(NULL);
-        } else if (delay_tidy) {
-            tv.tv_sec = 1;
-            last_tidy_cache = time(NULL) - idle_time + 2;
-            delay_tidy = 0;
-        } else if (dav_tidy_cache() == 0) {
+        } else {
+            dav_tidy_cache();
             tv.tv_sec = idle_time;
             last_tidy_cache = time(NULL);
-        } else {
-            tv.tv_sec = 0;
         }
 
         if (flush) {
@@ -356,7 +369,11 @@ static uint32_t coda_access(void) {
     struct coda_access_in *in = (struct coda_access_in *) buf;
     dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
-    DBG2("  n %p, f %x", node, in->flags);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_ACCESS:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n %p, f %x",
+               node, in->flags);
+    }
 
     int how = (in->flags & C_A_R_OK) ? R_OK : 0;
     how |= (in->flags & C_A_W_OK) ? W_OK : 0;
@@ -365,7 +382,6 @@ static uint32_t coda_access(void) {
     
     oh->result = dav_access(node, ih->uid, how);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_out_hdr);
 }
 
@@ -376,8 +392,17 @@ static uint32_t coda_close(void) {
     struct coda_close_in *in = (struct coda_close_in *) buf;
     dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
-    DBG2("  n %p, f %x", node, in->flags);
-    DBG2("  pid %i, pgid %i", ih->pid, ih->pgid);
+    if (debug) {
+        if (ih->opcode == CODA_CLOSE) {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_CLOSE:");
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_RELEASE:");
+        }
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n %p, f %x",
+               node, in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, pgid %i",
+               ih->pid, ih->pgid);
+    }
 
     int flags = 0;
     if ((in->flags & C_O_READ) && (in->flags & C_O_WRITE)) {
@@ -390,7 +415,6 @@ static uint32_t coda_close(void) {
 
     oh->result = dav_close(node, 0, flags, ih->pid, ih->pgid);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_out_hdr);
 }
 
@@ -402,8 +426,12 @@ static uint32_t coda_create(void) {
     dav_node *parent = *((dav_node **) &(in->VFid.opaque[2]));
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_create_out *out = (struct coda_create_out *) buf;
-    DBG2("  p %p, m %o", parent, in->mode);
-    DBG1("  %s", buf + in->name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_CREATE:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p %p, m %o",
+               parent, in->mode);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  %s", buf + in->name);
+    }
 
     dav_node *node = NULL;
     oh->result = dav_create(&node, parent, buf + in->name, ih->uid,
@@ -412,7 +440,6 @@ static uint32_t coda_create(void) {
     if (oh->result != 0 || node == NULL) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
@@ -422,7 +449,6 @@ static uint32_t coda_create(void) {
     *((dav_node **) &(out->VFid.opaque[2])) = node;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_create_out);
 }
 
@@ -434,11 +460,13 @@ static uint32_t coda_getattr(void) {
     dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_getattr_out *out = (struct coda_getattr_out *) buf;
-    DBG1("  n %p", node);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_GETATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n %p", node);
+    }
 
     oh->result = dav_getattr(node, ih->uid);
 
-    DBG_RET(oh->result);
     if (oh->result != 0)
         return sizeof(struct coda_out_hdr);
 
@@ -455,7 +483,11 @@ static uint32_t coda_lookup(void) {
     dav_node *parent = *((dav_node **) &(in->VFid.opaque[2]));
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_lookup_out *out = (struct coda_lookup_out *) buf;
-    DBG2("  p %p, %s", parent, buf + in->name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_LOOKUP:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p %p, %s", parent,
+               buf + in->name);
+    }
 
     dav_node *node = NULL;
     oh->result = dav_lookup(&node, parent, buf + in->name, ih->uid);
@@ -463,7 +495,6 @@ static uint32_t coda_lookup(void) {
     if (oh->result != 0 || node == NULL) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
@@ -473,7 +504,6 @@ static uint32_t coda_lookup(void) {
     *((dav_node **) &(out->VFid.opaque[2])) = node;
     out->vtype = (node->mode & S_IFDIR) ? CDT_DIR : CDT_REG;
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_lookup_out);
 }
 
@@ -485,7 +515,11 @@ static uint32_t coda_mkdir(void) {
     dav_node *parent = *((dav_node **) &(in->VFid.opaque[2]));
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_mkdir_out *out = (struct coda_mkdir_out *) buf;
-    DBG2("  p %p, %s", parent, buf + in->name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_MKDIR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p %p, %s", parent,
+               buf + in->name);
+    }
 
     dav_node *node = NULL;
     oh->result = dav_mkdir(&node, parent, buf + in->name, ih->uid,
@@ -494,7 +528,6 @@ static uint32_t coda_mkdir(void) {
     if (oh->result != 0 || node == NULL) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
@@ -504,7 +537,6 @@ static uint32_t coda_mkdir(void) {
     *((dav_node **) &(out->VFid.opaque[2])) = node;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_mkdir_out);
 }
 
@@ -516,8 +548,13 @@ static uint32_t coda_open_by_fd(void) {
     dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_open_by_fd_out *out = (struct coda_open_by_fd_out *) buf;
-    DBG2("  n %p, f %x", node, in->flags);
-    DBG2("  pid %i, pgid %i", ih->pid, ih->pgid);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_OPEN_BY_FD:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n %p, f %x", node,
+               in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, pgid %i",
+               ih->pid, ih->pgid);
+    }
 
     int flags = 0;
     if ((in->flags & C_O_READ) && (in->flags & C_O_WRITE)) {
@@ -534,11 +571,9 @@ static uint32_t coda_open_by_fd(void) {
     if (oh->result != 0 || out->fd == 0) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_open_by_fd_out);
 }
 
@@ -548,6 +583,8 @@ static uint32_t coda_root(void) {
     struct coda_in_hdr *ih = (struct coda_in_hdr *) buf;
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_root_out *out = (struct coda_root_out *) buf;
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_ROOT:");
 
     dav_node *node = NULL;
     oh->result = dav_root(&node, ih->uid);
@@ -555,7 +592,6 @@ static uint32_t coda_root(void) {
     if (oh->result != 0 || node == NULL) {
         if (oh->result == 0)
             oh->result = EIO;
-        DBG_RET(oh->result);
         return sizeof(struct coda_out_hdr);
     }
 
@@ -564,7 +600,6 @@ static uint32_t coda_root(void) {
     out->VFid.opaque[3] = 0;
     *((dav_node **) &(out->VFid.opaque[2])) = node;
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_root_out);
 }
 
@@ -575,11 +610,17 @@ static uint32_t coda_setattr(void) {
     struct coda_setattr_in *in = (struct coda_setattr_in *) buf;
     dav_node *node = *((dav_node **) &(in->VFid.opaque[2]));
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
-    DBG2("  n %p, m %o", node, in->attr.va_mode);
-    DBG2("  uid: %i, gid: %i", in->attr.va_uid, in->attr.va_gid);
-    DBG2("  at %li, mt %li", in->attr.va_atime.tv_sec,
-         in->attr.va_mtime.tv_sec);
-    DBG2("  ct %li, sz %llu", in->attr.va_ctime.tv_sec, in->attr.va_size);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_SETATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n %p, m %o", node,
+               in->attr.va_mode);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  uid: %i, gid: %i",
+               in->attr.va_uid, in->attr.va_gid);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  at %li, mt %li",
+               in->attr.va_atime.tv_sec, in->attr.va_mtime.tv_sec);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  ct %li, sz %llu",
+               in->attr.va_ctime.tv_sec, in->attr.va_size);
+    }
 
     oh->result = dav_setattr(node, ih->uid, in->attr.va_mode != USHRT_MAX,
                              in->attr.va_mode & DAV_A_MASK,
@@ -592,7 +633,6 @@ static uint32_t coda_setattr(void) {
                              in->attr.va_size != UINT64_MAX,
                              in->attr.va_size);
 
-    DBG_RET(oh->result);
     return sizeof(struct coda_out_hdr);
 }
 
@@ -601,6 +641,8 @@ static uint32_t coda_statfs(void) {
 
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
     struct coda_statfs_out *out = (struct coda_statfs_out *) buf;
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "CODA_STATFS:");
 
     dav_stat st = dav_statfs();
 
@@ -611,7 +653,6 @@ static uint32_t coda_statfs(void) {
     out->stat.f_ffree = st.ffree;
 
     oh->result = 0;
-    DBG_RET(oh->result);
     return sizeof(struct coda_statfs_out);
 }
 
@@ -622,7 +663,8 @@ static uint32_t coda_statfs(void) {
 static void coda_flush(int device) {
 
     struct coda_out_hdr *oh = (struct coda_out_hdr *) buf;
-    DBG0("  CODA_FLUSH:");
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  CODA_FLUSH:");
 
     oh->opcode = CODA_FLUSH;
     oh->unique = 0;
