@@ -42,7 +42,6 @@
 #   define _(String) String
 #endif
 
-#include "dav_debug.h"
 #include "defaults.h"
 #include "cache.h"
 #include "kernel_interface.h"
@@ -60,6 +59,9 @@ static char *buf;
    between the real nodeid and what fuse wants. */
 static uint64_t root;
 
+/* Send debug messages to syslog if != 0. */
+int debug;
+
 
 /* Private function prototypes */
 /*=============================*/
@@ -103,7 +105,11 @@ static void set_attr(struct fuse_attr *attr, const dav_node *node);
 
 void dav_fuse5_loop(int device, size_t bufsize, time_t idle_time,
                     dav_is_mounted_fn is_mounted,
-                    volatile int *keep_on_running) {
+                    volatile int *keep_on_running, int dbg) {
+
+    debug = dbg;
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "fuse kernel version 5");
 
     buf_size = bufsize;
     buf = malloc(buf_size);
@@ -119,7 +125,6 @@ void dav_fuse5_loop(int device, size_t bufsize, time_t idle_time,
     tv.tv_sec = idle_time;
     tv.tv_usec = 0;
     time_t last_tidy_cache = time(NULL);
-    int delay_tidy = 0;
 
     while (*keep_on_running) {
 
@@ -131,6 +136,9 @@ void dav_fuse5_loop(int device, size_t bufsize, time_t idle_time,
         if (ret > 0) {
             ssize_t bytes_read = read(device, buf, buf_size);
             if (bytes_read <= 0) {
+                if (debug)
+                    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "READ: %s",
+                           strerror(errno));
                 if (bytes_read == 0 || errno == EINTR || errno == ENOENT) {
                     if (time(NULL) < (last_tidy_cache + idle_time)) {
                         tv.tv_sec = last_tidy_cache + idle_time - time(NULL);
@@ -162,150 +170,159 @@ void dav_fuse5_loop(int device, size_t bufsize, time_t idle_time,
 
         switch (ih->opcode) {
         case FUSE_LOOKUP:
-            DBG0("FUSE_LOOKUP:");
             oh->len = fuse_lookup();
             break;
         case FUSE_FORGET:
-            DBG0("FUSE_FORGET: no reply");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "FUSE_FORGET: no reply");
+            oh->error = 0;
             oh->len = 0;
             break;
         case FUSE_GETATTR:
-            DBG0("FUSE_GETATTR:");
             oh->len = fuse_getattr();
             break;
         case FUSE_SETATTR:
-            DBG0("FUSE_SETATTR:");
             oh->len = fuse_setattr();
             break;
         case FUSE_READLINK:
-            DBG0("FUSE_READLINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_READLINK:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_SYMLINK:
-            DBG0("FUSE_SYMLINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_SYMLINK:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_MKNOD:
-            DBG0("FUSE_MKNOD:");
             oh->len = fuse_mknod();
             break;
         case FUSE_MKDIR:
-            DBG0("FUSE_MKDIR:");
             oh->len = fuse_mkdir();
             break;
         case FUSE_UNLINK:
-            DBG0("FUSE_UNLINK:");
-            DBG2("  p 0x%llx, %s", ih->nodeid,
-                 (char *) (buf + sizeof(struct fuse_in_header)));
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_UNLINK:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, %s",
+                       ih->nodeid,
+                       (char *) (buf + sizeof(struct fuse_in_header)));
+            }
             oh->error = dav_remove((dav_node *) ((size_t) ih->nodeid),
                              (char *) (buf + sizeof(struct fuse_in_header)),
                              ih->uid);
-            DBG_RET(oh->error);
             if (oh->error != 0)
                 oh->error *= -1;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_RMDIR:
-            DBG0("FUSE_RMDIR:");
-            DBG2("  p 0x%llx, %s", ih->nodeid,
-                 (char *) (buf + sizeof(struct fuse_in_header)));
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_RMDIR:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, %s",
+                       ih->nodeid,
+                       (char *) (buf + sizeof(struct fuse_in_header)));
+            }
             oh->error = dav_rmdir((dav_node *) ((size_t) ih->nodeid),
                             (char *) (buf + sizeof(struct fuse_in_header)),
                             ih->uid);
-            DBG_RET(oh->error);
             if (oh->error != 0)
                 oh->error *= -1;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_RENAME:
-            DBG0("FUSE_RENAME:");
             oh->len = fuse_rename();
             break;
         case FUSE_LINK:
-            DBG0("FUSE_LINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_LINK:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_OPEN:
-            DBG0("FUSE_OPEN:");
             oh->len = fuse_open();
             break;
         case FUSE_READ:
-            DBG0("FUSE_READ:");
             oh->len = fuse_read();
             break;
         case FUSE_WRITE:
-            DBG0("FUSE_WRITE:");
             oh->len = fuse_write();
             break;
         case FUSE_STATFS:
-            DBG0("FUSE_STATFS:");
             oh->len = fuse_stat();
             break;
         case FUSE_RELEASE:
-            DBG0("FUSE_RELEASE:");
             oh->len = fuse_release();
-            delay_tidy = 1;
+            last_tidy_cache = 0;
             break;
         case FUSE_FSYNC:
-            DBG0("FUSE_FSYNC:");
-            DBG1("  n 0x%llx", ih->nodeid);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_FSYNC:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx",
+                ih->nodeid);
+            }
             oh->error = dav_sync((dav_node *) ((size_t) ih->nodeid));
-            DBG_RET(oh->error);
             if (oh->error != 0)
                 oh->error *= -1;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_SETXATTR:
-            DBG0("FUSE_SETXATTR: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_SETXATTR:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_GETXATTR:
-            DBG0("FUSE_GETXATTR: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_GETXATTR:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_LISTXATTR:
-            DBG0("FUSE_LISTXATTR: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_LISTXATTR:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_REMOVEXATTR:
-            DBG0("FUSE_REMOVEXATTR: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_REMOVEXATTR:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_FLUSH:
-            DBG0("FUSE_FLUSH: ignored");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "FUSE_FLUSH: ignored");
             oh->error = 0;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_INIT:
-            DBG0("FUSE_INIT:");
             oh->len = fuse_init();
             break;
         case FUSE_OPENDIR:
-            DBG0("FUSE_OPENDIR:");
             oh->len = fuse_open();
             break;
         case FUSE_READDIR:
-            DBG0("FUSE_READDIR:");
             oh->len = fuse_read();
             break;
         case FUSE_RELEASEDIR:
-            DBG0("FUSE_RELEASEDIR:");
             oh->len = fuse_release();
             break;
         default:
-            DBG1("UNKNOWN FUSE CALL %i", ih->opcode);
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "UNKNOWN FUSE CALL %i", ih->opcode);
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         }
 
+        if (debug && oh->len)
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "RET: %s",
+                   strerror(-oh->error));
+
         ssize_t n = 0;
         ssize_t w = 0;
         while (n < oh->len && w >= 0) {
@@ -315,15 +332,10 @@ void dav_fuse5_loop(int device, size_t bufsize, time_t idle_time,
 
         if (time(NULL) < (last_tidy_cache + idle_time)) {
             tv.tv_sec = last_tidy_cache + idle_time - time(NULL);
-        } else if (delay_tidy) {
-            tv.tv_sec = 1;
-            last_tidy_cache = time(NULL) - idle_time + 2;
-            delay_tidy = 0;
-        } else if (dav_tidy_cache() == 0) {
+        } else {
+            dav_tidy_cache();
             tv.tv_sec = idle_time;
             last_tidy_cache = time(NULL);
-        } else {
-            tv.tv_sec = 0;
         }
 
     }
@@ -347,11 +359,13 @@ static uint32_t fuse_getattr(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_attr_out *out = (struct fuse_attr_out *)
                                 (buf + sizeof(struct fuse_out_header));
-    DBG1("  n 0x%llx", ih->nodeid);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_GETATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx", ih->nodeid);
+    }
 
     oh->error = dav_getattr((dav_node *) ((size_t) ih->nodeid), ih->uid);
 
-    DBG_RET(oh->error);
     if (oh->error != 0) {
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
@@ -372,11 +386,12 @@ static uint32_t fuse_init(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_init_in_out *out = (struct fuse_init_in_out *)
                                    (buf + sizeof(struct fuse_out_header));
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_INIT:");
 
     dav_node *node;
     oh->error = dav_root(&node, ih->uid);
 
-    DBG_RET(oh->error);
     if (oh->error != 0 || node == NULL) {
         if (oh->error == 0)
             oh->error = EIO;
@@ -399,7 +414,11 @@ static uint32_t fuse_lookup(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_entry_out *out = (struct fuse_entry_out *)
                                  (buf + sizeof(struct fuse_out_header));
-    DBG2("  p 0x%llx, %s", ih->nodeid, name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_LOOKUP:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, %s",
+               ih->nodeid, name);
+    }
 
     dav_node *node = NULL;
     oh->error = dav_lookup(&node, (dav_node *) ((size_t) ih->nodeid), name,
@@ -408,7 +427,6 @@ static uint32_t fuse_lookup(void) {
     if (oh->error != 0 || node == NULL) {
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -421,7 +439,6 @@ static uint32_t fuse_lookup(void) {
     out->attr_valid_nsec = 0;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->error);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_entry_out);
 }
 
@@ -436,7 +453,11 @@ static uint32_t fuse_mkdir(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_entry_out *out = (struct fuse_entry_out *)
                                  (buf + sizeof(struct fuse_out_header));
-    DBG2("  p 0x%llx, %s", ih->nodeid, name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_MKDIR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, %s",
+               ih->nodeid, name);
+    }
 
     dav_node *node = NULL;
     oh->error = dav_mkdir(&node, (dav_node *) ((size_t) ih->nodeid), name,
@@ -445,7 +466,6 @@ static uint32_t fuse_mkdir(void) {
     if (oh->error != 0 || node == NULL) {
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -458,7 +478,6 @@ static uint32_t fuse_mkdir(void) {
     out->attr_valid_nsec = 0;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->error);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_entry_out);
 }
 
@@ -473,11 +492,14 @@ static uint32_t fuse_mknod(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_entry_out *out = (struct fuse_entry_out *)
                                  (buf + sizeof(struct fuse_out_header));
-    DBG2("  p 0x%llx, m 0%o", ih->nodeid, in->mode);
-    DBG1("  %s", name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_MKNOD:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, m 0%o",
+               ih->nodeid, in->mode);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  %s", name);
+    }
 
     if (!S_ISREG(in->mode)) {
-        DBG_RET(ENOTSUP);
         oh->error = -ENOTSUP;
         return sizeof(struct fuse_out_header);
     }
@@ -489,7 +511,6 @@ static uint32_t fuse_mknod(void) {
     if (oh->error != 0 || node == NULL) {
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -502,7 +523,6 @@ static uint32_t fuse_mknod(void) {
     out->attr_valid_nsec = 0;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->error);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_entry_out);
 }
 
@@ -515,8 +535,16 @@ static uint32_t fuse_open(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_open_out *out = (struct fuse_open_out *)
                                 (buf + sizeof(struct fuse_out_header));
-    DBG2("  n 0x%llx, f 0%o", ih->nodeid, in->flags);
-    DBG1("  pid %i", ih->pid);
+    if (debug) {
+        if (ih->opcode == FUSE_OPENDIR) {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_OPENDIR:");
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_OPEN:");
+        }
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, f 0%o",
+               ih->nodeid, in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i", ih->pid);
+    }
 
     int fd = 0;
     oh->error = dav_open(&fd, (dav_node *) ((size_t) ih->nodeid),
@@ -525,7 +553,6 @@ static uint32_t fuse_open(void) {
     if (oh->error != 0 || fd == 0) {
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -533,8 +560,8 @@ static uint32_t fuse_open(void) {
     out->open_flags = in->flags & (O_ACCMODE | O_APPEND);
     out->fh = fd;
 
-    DBG1("  fd %i", fd);
-    DBG_RET(oh->error);
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  fd %i", fd);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_open_out);
 }
 
@@ -545,12 +572,20 @@ static uint32_t fuse_read(void) {
     struct fuse_read_in *in = (struct fuse_read_in *)
                               (buf + sizeof(struct fuse_in_header));
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
-    DBG2("  n 0x%llx, fd %llu", ih->nodeid, in->fh);
-    DBG1("  pid %i", ih->pid);
-    DBG2("  size %u, off %llu", in->size, in->offset);
+    if (debug) {
+        if (ih->opcode == FUSE_READDIR) {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_READDIR:");
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_READ:");
+        }
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, fd %llu",
+               ih->nodeid, in->fh);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i", ih->pid);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  size %u, off %llu",
+               in->size, in->offset);
+    }
 
     if (in->size > (buf_size - sizeof(struct fuse_out_header))) {
-        DBG_RET(EINVAL);
         oh->error = -EINVAL;
         return sizeof(struct fuse_out_header);
     }
@@ -560,7 +595,6 @@ static uint32_t fuse_read(void) {
                          in->fh, buf + sizeof(struct fuse_out_header),
                          in->size, in->offset);
 
-    DBG_RET(oh->error);
     if (oh->error != 0)
         oh->error *= -1;
 
@@ -574,13 +608,21 @@ static uint32_t fuse_release(void) {
     struct fuse_release_in *in = (struct fuse_release_in *)
                                  (buf + sizeof(struct fuse_in_header));
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
-    DBG2("  n 0x%llx, f 0%o", ih->nodeid, in->flags);
-    DBG2("  pid %i, fd %llu", ih->pid, in->fh);
+    if (debug) {
+        if (ih->opcode == FUSE_RELEASEDIR) {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_RELEASEDIR:");
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_RELEASE:");
+        }
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, f 0%o",
+               ih->nodeid, in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, fd %llu",
+               ih->pid, in->fh);
+    }
 
     oh->error = dav_close((dav_node *) ((size_t) ih->nodeid), in->fh,
                           in->flags, ih->pid, 0);
 
-    DBG_RET(oh->error);
     if (oh->error != 0)
         oh->error *= -1;
 
@@ -597,15 +639,19 @@ static uint32_t fuse_rename(void) {
                           + sizeof(struct fuse_rename_in));
     char *new = old + strlen(old) + 1;
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
-    DBG2("  sp 0x%llx, %s", ih->nodeid, old);
-    DBG2("  dp 0x%llx, %s", in->newdir, new);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_RENAME:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  sp 0x%llx, %s",
+               ih->nodeid, old);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  dp 0x%llx, %s",
+               in->newdir, new);
+    }
 
     if (in->newdir == 1)
         in->newdir = root;
     oh->error = dav_rename((dav_node *) ((size_t) ih->nodeid), old,
                            (dav_node *) ((size_t) in->newdir), new, ih->uid);
 
-    DBG_RET(oh->error);
     if (oh->error != 0)
         oh->error *= -1;
 
@@ -621,10 +667,17 @@ static uint32_t fuse_setattr(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_attr_out *out = (struct fuse_attr_out *)
                                 (buf + sizeof(struct fuse_out_header));
-    DBG2("  n 0x%llx, m 0%o", ih->nodeid, in->attr.mode);
-    DBG2("  uid %i, gid %i", in->attr.uid, in->attr.gid);
-    DBG2("  sz %llu, at %llu,", in->attr.size, in->attr.atime);
-    DBG2("  mt %llu, ct %llu", in->attr.mtime, in->attr.ctime);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_SETATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, m 0%o",
+               ih->nodeid, in->attr.mode);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  uid %i, gid %i",
+               in->attr.uid, in->attr.gid);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  sz %llu, at %llu,",
+               in->attr.size, in->attr.atime);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  mt %llu, ct %llu",
+               in->attr.mtime, in->attr.ctime);
+    }
 
     oh->error = dav_setattr((dav_node *) ((size_t) ih->nodeid), ih->uid,
                             in->valid & FATTR_MODE, in->attr.mode,
@@ -634,7 +687,6 @@ static uint32_t fuse_setattr(void) {
                             in->valid & FATTR_MTIME, in->attr.mtime,
                             in->valid & FATTR_SIZE, in->attr.size);
 
-    DBG_RET(oh->error);
     if (oh->error != 0) {
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
@@ -654,6 +706,8 @@ static uint32_t fuse_stat(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_statfs_out *out = (struct fuse_statfs_out *)
                                   (buf + sizeof(struct fuse_out_header));
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_STATFS:");
 
     dav_stat st = dav_statfs();
 
@@ -670,7 +724,6 @@ static uint32_t fuse_stat(void) {
     out->st.namelen = st.namelen;
 
     oh->error = 0;
-    DBG_RET(oh->error);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_statfs_out);
 }
 
@@ -683,13 +736,18 @@ static uint32_t fuse_write(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_write_out *out = (struct fuse_write_out *)
                                  (buf + sizeof(struct fuse_out_header));
-    DBG2("  n 0x%llx, fd %llu", ih->nodeid, in->fh);
-    DBG2("  pid %i, flags 0%o", ih->pid, in->write_flags);
-    DBG2("  size %u, off %llu", in->size, in->offset);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_WRITE:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, fd %llu",
+               ih->nodeid, in->fh);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, flags 0%o",
+               ih->pid, in->write_flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  size %u, off %llu",
+               in->size, in->offset);
+    }
 
     if (in->size > (buf_size - sizeof(struct fuse_in_header)
                     - sizeof(struct fuse_write_in))) {
-        DBG_RET(EINVAL);
         oh->error = -EINVAL;
         return sizeof(struct fuse_out_header);
     }
@@ -700,7 +758,6 @@ static uint32_t fuse_write(void) {
                           + sizeof(struct fuse_write_in),
                           in->size, in->offset);
 
-    DBG_RET(oh->error);
     if (oh->error != 0) {
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
