@@ -42,7 +42,6 @@
 #   define _(String) String
 #endif
 
-#include "dav_debug.h"
 #include "defaults.h"
 #include "cache.h"
 #include "kernel_interface.h"
@@ -71,6 +70,9 @@ static char *buf;
    between the real nodeid and what fuse wants. */
 static uint64_t root;
 
+/* Send debug messages to syslog if != 0. */
+int debug;
+
 
 /* Private function prototypes */
 /*=============================*/
@@ -83,6 +85,8 @@ static uint32_t fuse_create(void);
 
 static uint32_t fuse_getattr(void);
 
+static uint32_t fuse_getxattr(void);
+
 static uint32_t fuse_init(void);
 
 static uint32_t fuse_lookup(void);
@@ -118,7 +122,11 @@ static void set_attr(struct fuse_attr *attr, const dav_node *node);
 
 void dav_fuse7_loop(int device, size_t bufsize, time_t idle_time,
                     dav_is_mounted_fn is_mounted,
-                    volatile int *keep_on_running) {
+                    volatile int *keep_on_running, int dbg) {
+
+    debug = dbg;
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "fuse kernel version 7");
 
     buf_size = bufsize;
     buf = malloc(buf_size);
@@ -134,7 +142,6 @@ void dav_fuse7_loop(int device, size_t bufsize, time_t idle_time,
     tv.tv_sec = idle_time;
     tv.tv_usec = 0;
     time_t last_tidy_cache = time(NULL);
-    int delay_tidy = 0;
 
     while (*keep_on_running) {
 
@@ -142,10 +149,15 @@ void dav_fuse7_loop(int device, size_t bufsize, time_t idle_time,
         FD_ZERO(&fds);
         FD_SET(device, &fds);
         int ret = select(device + 1, &fds, NULL, NULL, &tv);
+        if (debug)
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "SELECT: %i", ret);
 
         if (ret > 0) {
             ssize_t bytes_read = read(device, buf, buf_size);
             if (bytes_read <= 0) {
+                if (debug)
+                    syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "READ: %s",
+                           strerror(errno));
                 if (bytes_read == 0 || errno == EINTR || errno == EAGAIN ||
                         errno == ENOENT) {
                     if (time(NULL) < (last_tidy_cache + idle_time)) {
@@ -178,163 +190,168 @@ void dav_fuse7_loop(int device, size_t bufsize, time_t idle_time,
 
         switch (ih->opcode) {
         case FUSE_LOOKUP:
-            DBG0("FUSE_LOOKUP:");
             oh->len = fuse_lookup();
             break;
         case FUSE_FORGET:
-            DBG0("FUSE_FORGET: no reply");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "FUSE_FORGET: no reply");
+            oh->error = 0;
             oh->len = 0;
             break;
         case FUSE_GETATTR:
-            DBG0("FUSE_GETATTR:");
             oh->len = fuse_getattr();
             break;
         case FUSE_SETATTR:
-            DBG0("FUSE_SETATTR:");
             oh->len = fuse_setattr();
             break;
         case FUSE_READLINK:
-            DBG0("FUSE_READLINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_READLINK:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_SYMLINK:
-            DBG0("FUSE_SYMLINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_SYMLINK:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_MKNOD:
-            DBG0("FUSE_MKNOD:");
             oh->len = fuse_mknod();
             break;
         case FUSE_MKDIR:
-            DBG0("FUSE_MKDIR:");
             oh->len = fuse_mkdir();
             break;
         case FUSE_UNLINK:
-            DBG0("FUSE_UNLINK:");
-            DBG2("  p 0x%llx, %s", ih->nodeid,
-                 (char *) (buf + sizeof(struct fuse_in_header)));
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_UNLINK:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, %s",
+                       ih->nodeid,
+                       (char *) (buf + sizeof(struct fuse_in_header)));
+            }
             oh->error = dav_remove((dav_node *) ((size_t) ih->nodeid),
                              (char *) (buf + sizeof(struct fuse_in_header)),
                              ih->uid);
-            DBG_RET(oh->error);
             if (oh->error != 0)
                 oh->error *= -1;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_RMDIR:
-            DBG0("FUSE_RMDIR:");
-            DBG2("  p 0x%llx, %s", ih->nodeid,
-                 (char *) (buf + sizeof(struct fuse_in_header)));
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_RMDIR:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, %s",
+                       ih->nodeid,
+                       (char *) (buf + sizeof(struct fuse_in_header)));
+            }
             oh->error = dav_rmdir((dav_node *) ((size_t) ih->nodeid),
                             (char *) (buf + sizeof(struct fuse_in_header)),
                             ih->uid);
-            DBG_RET(oh->error);
             if (oh->error != 0)
                 oh->error *= -1;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_RENAME:
-            DBG0("FUSE_RENAME:");
             oh->len = fuse_rename();
             break;
         case FUSE_LINK:
-            DBG0("FUSE_LINK: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_LINK:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_OPEN:
-            DBG0("FUSE_OPEN:");
             oh->len = fuse_open();
             break;
         case FUSE_READ:
-            DBG0("FUSE_READ:");
             oh->len = fuse_read();
             break;
         case FUSE_WRITE:
-            DBG0("FUSE_WRITE:");
             oh->len = fuse_write();
             break;
         case FUSE_STATFS:
-            DBG0("FUSE_STATFS:");
             oh->len = fuse_stat();
             break;
         case FUSE_RELEASE:
-            DBG0("FUSE_RELEASE:");
             oh->len = fuse_release();
-            delay_tidy = 1;
+            last_tidy_cache = 0;
             break;
         case FUSE_FSYNC:
-            DBG0("FUSE_FSYNC:");
-            DBG1("  n 0x%llx", ih->nodeid);
+            if (debug) {
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_FSYNC:");
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx",
+                ih->nodeid);
+            }
             oh->error = dav_sync((dav_node *) ((size_t) ih->nodeid));
-            DBG_RET(oh->error);
             if (oh->error != 0)
                 oh->error *= -1;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_SETXATTR:
-            DBG0("FUSE_SETXATTR: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_GETXATTR:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_GETXATTR:
-            DBG0("FUSE_GETXATTR: not supported");
-            oh->error = -ENOTSUP;
-            oh->len = sizeof(struct fuse_out_header);
+            oh->len = fuse_getxattr();
             break;
         case FUSE_LISTXATTR:
-            DBG0("FUSE_LISTXATTR: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_LISTXATTR:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_REMOVEXATTR:
-            DBG0("FUSE_REMOVEXATTR: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_REMOVEXATTR:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_FLUSH:
-            DBG0("FUSE_FLUSH: ignored");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "FUSE_FLUSH: ignored");
             oh->error = 0;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_INIT:
-            DBG0("FUSE_INIT:");
             oh->len = fuse_init();
             break;
         case FUSE_OPENDIR:
-            DBG0("FUSE_OPENDIR:");
             oh->len = fuse_open();
             break;
         case FUSE_READDIR:
-            DBG0("FUSE_READDIR:");
             oh->len = fuse_read();
             break;
         case FUSE_RELEASEDIR:
-            DBG0("FUSE_RELEASEDIR:");
             oh->len = fuse_release();
             break;
         case FUSE_FSYNCDIR:
-            DBG0("FUSE_FSYNCDIR: not supported");
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_FSYNCDIR:");
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         case FUSE_ACCESS:
-            DBG0("FUSE_ACCESS:");
             oh->len = fuse_access();
             break;
         case FUSE_CREATE:
-            DBG0("FUSE_CREATE:");
             oh->len = fuse_create();
             break;
         default:
-            DBG1("UNKNOWN FUSE CALL %i", ih->opcode);
+            if (debug)
+                syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+                       "UNKNOWN FUSE CALL %i", ih->opcode);
             oh->error = -ENOTSUP;
             oh->len = sizeof(struct fuse_out_header);
             break;
         }
 
+        if (debug && oh->len)
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "RET: %s",
+                   strerror(-oh->error));
+
         ssize_t n = 0;
         ssize_t w = 0;
         while (n < oh->len && w >= 0) {
@@ -344,15 +361,10 @@ void dav_fuse7_loop(int device, size_t bufsize, time_t idle_time,
 
         if (time(NULL) < (last_tidy_cache + idle_time)) {
             tv.tv_sec = last_tidy_cache + idle_time - time(NULL);
-        } else if (delay_tidy) {
-            tv.tv_sec = 1;
-            last_tidy_cache = time(NULL) - idle_time + 2;
-            delay_tidy = 0;
-        } else if (dav_tidy_cache() == 0) {
+        } else {
+            dav_tidy_cache();
             tv.tv_sec = idle_time;
             last_tidy_cache = time(NULL);
-        } else {
-            tv.tv_sec = 0;
         }
 
     }
@@ -376,13 +388,16 @@ static uint32_t fuse_access(void) {
     struct fuse_access_in *in = (struct fuse_access_in *)
                                 (buf + sizeof(struct fuse_in_header));
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
-    DBG2("  n 0x%llx, f 0%o", ih->nodeid, in->mask);
-    DBG1("  uid %i", ih->uid);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_ACCESS:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, f 0%o",
+               ih->nodeid, in->mask);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  uid %i", ih->uid);
+    }
 
     oh->error = dav_access((dav_node *) ((size_t) ih->nodeid), ih->uid,
                            in->mask);
 
-    DBG_RET(oh->error);
     if (oh->error != 0)
         oh->error *= -1;
 
@@ -400,8 +415,13 @@ static uint32_t fuse_create(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct create_out *out = (struct create_out *)
                              (buf + sizeof(struct fuse_out_header));
-    DBG2("  n 0x%llx, f 0%o", ih->nodeid, in->flags);
-    DBG2("  pid %i, mode 0%o", ih->pid, in->mode);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_CREATE:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, f 0%o",
+               ih->nodeid, in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, mode 0%o",
+               ih->pid, in->mode);
+    }
 
     int created = 0;
     dav_node *node = NULL;
@@ -411,11 +431,9 @@ static uint32_t fuse_create(void) {
     if (oh->error == 0 ) {
         if (node == NULL) {
             oh->error = -EIO;
-            DBG_RET(EIO);
             return sizeof(struct fuse_out_header);
         } else if (in->flags & O_EXCL) {
             oh->error = -EEXIST;
-            DBG_RET(EEXIST);
             return sizeof(struct fuse_out_header);
         }
     } else if (oh->error == ENOENT) {
@@ -424,13 +442,11 @@ static uint32_t fuse_create(void) {
         if (oh->error != 0 || node == NULL) {
             if (oh->error == 0)
                 oh->error = EIO;
-            DBG_RET(oh->error);
             oh->error *= -1;
             return sizeof(struct fuse_out_header);
         }
         created = 1;
     } else {
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -444,7 +460,6 @@ static uint32_t fuse_create(void) {
             dav_remove((dav_node *) ((size_t) ih->nodeid), name, ih->uid);
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -461,7 +476,8 @@ static uint32_t fuse_create(void) {
     out->open.fh = fd;
     out->open.padding = 0;
 
-    DBG_RET(oh->error);
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  fd %i", fd);
     return sizeof(struct fuse_out_header) + sizeof(struct create_out);
 }
 
@@ -472,11 +488,13 @@ static uint32_t fuse_getattr(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_attr_out *out = (struct fuse_attr_out *)
                                 (buf + sizeof(struct fuse_out_header));
-    DBG1("  n 0x%llx", ih->nodeid);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_GETATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx", ih->nodeid);
+    }
 
     oh->error = dav_getattr((dav_node *) ((size_t) ih->nodeid), ih->uid);
 
-    DBG_RET(oh->error);
     if (oh->error != 0) {
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
@@ -491,17 +509,66 @@ static uint32_t fuse_getattr(void) {
 }
 
 
+static uint32_t fuse_getxattr(void) {
+
+    struct fuse_in_header *ih = (struct fuse_in_header *) buf;
+    struct fuse_getxattr_in *in = (struct fuse_getxattr_in *)
+                                  (buf + sizeof(struct fuse_in_header));
+    char *name = (char *) (buf + sizeof(struct fuse_in_header)
+                           + sizeof(struct fuse_getxattr_in));
+    struct fuse_out_header *oh = (struct fuse_out_header *) buf;
+    struct fuse_getxattr_out *out = (struct fuse_getxattr_out *)
+                                    (buf + sizeof(struct fuse_out_header));
+    char *value = (char *) (buf + sizeof(struct fuse_out_header));
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_GETXATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, %s, %i",
+               ih->nodeid, name, in->size);
+    }
+
+    size_t size = in->size;
+    if (size == 0) {
+        oh->error = dav_getxattr((dav_node *) ((size_t) ih->nodeid), name,
+                                 value, &size, ih->uid);
+        if (oh->error != 0) {
+            oh->error *= -1;
+            return sizeof(struct fuse_out_header);
+        }
+        out->size = size;
+        out->padding = 0;
+        return sizeof(struct fuse_out_header)
+               + sizeof(struct fuse_getxattr_out);
+    } else {
+        if (size > (buf_size - sizeof(struct fuse_out_header)))
+            size = buf_size - sizeof(struct fuse_out_header);
+        oh->error = dav_getxattr((dav_node *) ((size_t) ih->nodeid), name,
+                                 value, &size, ih->uid);
+        if (oh->error != 0) {
+            oh->error *= -1;
+            return sizeof(struct fuse_out_header);
+        }
+        return sizeof(struct fuse_out_header) +  size;
+    }
+}
+
+
 static uint32_t fuse_init(void) {
 
     struct fuse_in_header *ih = (struct fuse_in_header *) buf;
+    struct fuse_init_in *in = (struct fuse_init_in *)
+                                  (buf + sizeof(struct fuse_in_header));
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_init_out *out = (struct fuse_init_out *)
                                 (buf + sizeof(struct fuse_out_header));
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_INIT:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  version %i.%i",
+               in->major, in->minor);
+    }
 
     dav_node *node;
     oh->error = dav_root(&node, ih->uid);
 
-    DBG_RET(oh->error);
     if (oh->error != 0 || node == NULL) {
         if (oh->error == 0)
             oh->error = EIO;
@@ -516,7 +583,7 @@ static uint32_t fuse_init(void) {
     out->unused[1] = 0;
     out->unused[2] = 0;
     out->max_write = buf_size - sizeof(struct fuse_in_header)
-                     - sizeof(struct fuse_write_in);
+                     - sizeof(struct fuse_write_in) - 4095;
 
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_init_out);
 }
@@ -529,7 +596,11 @@ static uint32_t fuse_lookup(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_entry_out *out = (struct fuse_entry_out *)
                                  (buf + sizeof(struct fuse_out_header));
-    DBG2("  p 0x%llx, %s", ih->nodeid, name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_LOOKUP:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, %s",
+               ih->nodeid, name);
+    }
 
     dav_node *node = NULL;
     oh->error = dav_lookup(&node, (dav_node *) ((size_t) ih->nodeid), name,
@@ -538,7 +609,6 @@ static uint32_t fuse_lookup(void) {
     if (oh->error != 0 || node == NULL) {
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -551,7 +621,6 @@ static uint32_t fuse_lookup(void) {
     out->attr_valid_nsec = 0;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->error);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_entry_out);
 }
 
@@ -566,7 +635,11 @@ static uint32_t fuse_mkdir(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_entry_out *out = (struct fuse_entry_out *)
                                  (buf + sizeof(struct fuse_out_header));
-    DBG2("  p 0x%llx, %s", ih->nodeid, name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_MKDIR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, %s",
+               ih->nodeid, name);
+    }
 
     dav_node *node = NULL;
     oh->error = dav_mkdir(&node, (dav_node *) ((size_t) ih->nodeid), name,
@@ -575,7 +648,6 @@ static uint32_t fuse_mkdir(void) {
     if (oh->error != 0 || node == NULL) {
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -588,7 +660,6 @@ static uint32_t fuse_mkdir(void) {
     out->attr_valid_nsec = 0;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->error);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_entry_out);
 }
 
@@ -603,11 +674,14 @@ static uint32_t fuse_mknod(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_entry_out *out = (struct fuse_entry_out *)
                                  (buf + sizeof(struct fuse_out_header));
-    DBG2("  p 0x%llx, m 0%o", ih->nodeid, in->mode);
-    DBG1("  %s", name);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_MKNOD:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  p 0x%llx, m 0%o",
+               ih->nodeid, in->mode);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  %s", name);
+    }
 
     if (!S_ISREG(in->mode)) {
-        DBG_RET(ENOTSUP);
         oh->error = -ENOTSUP;
         return sizeof(struct fuse_out_header);
     }
@@ -619,7 +693,6 @@ static uint32_t fuse_mknod(void) {
     if (oh->error != 0 || node == NULL) {
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -632,7 +705,6 @@ static uint32_t fuse_mknod(void) {
     out->attr_valid_nsec = 0;
     set_attr(&out->attr, node);
 
-    DBG_RET(oh->error);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_entry_out);
 }
 
@@ -645,8 +717,17 @@ static uint32_t fuse_open(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_open_out *out = (struct fuse_open_out *)
                                 (buf + sizeof(struct fuse_out_header));
-    DBG2("  n 0x%llx, f 0%o", ih->nodeid, in->flags);
-    DBG2("  pid %i, mode 0%o", ih->pid, in->mode);
+    if (debug) {
+        if (ih->opcode == FUSE_OPENDIR) {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_OPENDIR:");
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_OPEN:");
+        }
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, f 0%o",
+               ih->nodeid, in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, mode 0%o",
+               ih->pid, in->mode);
+    }
 
     int fd = 0;
     oh->error = dav_open(&fd, (dav_node *) ((size_t) ih->nodeid), in->flags,
@@ -655,7 +736,6 @@ static uint32_t fuse_open(void) {
     if (oh->error != 0 || fd == 0) {
         if (oh->error == 0)
             oh->error = EIO;
-        DBG_RET(oh->error);
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
     }
@@ -664,8 +744,8 @@ static uint32_t fuse_open(void) {
     out->fh = fd;
     out->padding = 0;
 
-    DBG1("  fd %i", fd);
-    DBG_RET(oh->error);
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  fd %i", fd);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_open_out);
 }
 
@@ -676,12 +756,20 @@ static uint32_t fuse_read(void) {
     struct fuse_read_in *in = (struct fuse_read_in *)
                               (buf + sizeof(struct fuse_in_header));
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
-    DBG2("  n 0x%llx, fd %llu", ih->nodeid, in->fh);
-    DBG1("  pid %i", ih->pid);
-    DBG2("  size %u, off %llu", in->size, in->offset);
+    if (debug) {
+        if (ih->opcode == FUSE_READDIR) {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_READDIR:");
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_READ:");
+        }
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, fd %llu",
+               ih->nodeid, in->fh);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i", ih->pid);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  size %u, off %llu",
+               in->size, in->offset);
+    }
 
     if (in->size > (buf_size - sizeof(struct fuse_out_header))) {
-        DBG_RET(EINVAL);
         oh->error = -EINVAL;
         return sizeof(struct fuse_out_header);
     }
@@ -691,7 +779,6 @@ static uint32_t fuse_read(void) {
                          in->fh, buf + sizeof(struct fuse_out_header),
                          in->size, in->offset);
 
-    DBG_RET(oh->error);
     if (oh->error != 0)
         oh->error *= -1;
 
@@ -705,13 +792,21 @@ static uint32_t fuse_release(void) {
     struct fuse_release_in *in = (struct fuse_release_in *)
                                  (buf + sizeof(struct fuse_in_header));
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
-    DBG2("  n 0x%llx, f 0%o", ih->nodeid, in->flags);
-    DBG2("  pid %i, fd %llu", ih->pid, in->fh);
+    if (debug) {
+        if (ih->opcode == FUSE_RELEASEDIR) {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_RELEASEDIR:");
+        } else {
+            syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_RELEASE:");
+        }
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, f 0%o",
+               ih->nodeid, in->flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, fd %llu",
+               ih->pid, in->fh);
+    }
 
     oh->error = dav_close((dav_node *) ((size_t) ih->nodeid), in->fh,
                           in->flags, ih->pid, 0);
 
-    DBG_RET(oh->error);
     if (oh->error != 0)
         oh->error *= -1;
 
@@ -728,15 +823,19 @@ static uint32_t fuse_rename(void) {
                           + sizeof(struct fuse_rename_in));
     char *new = old + strlen(old) + 1;
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
-    DBG2("  sp 0x%llx, %s", ih->nodeid, old);
-    DBG2("  dp 0x%llx, %s", in->newdir, new);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_RENAME:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  sp 0x%llx, %s",
+               ih->nodeid, old);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  dp 0x%llx, %s",
+               in->newdir, new);
+    }
 
     if (in->newdir == 1)
         in->newdir = root;
     oh->error = dav_rename((dav_node *) ((size_t) ih->nodeid), old,
                            (dav_node *) ((size_t) in->newdir), new, ih->uid);
 
-    DBG_RET(oh->error);
     if (oh->error != 0)
         oh->error *= -1;
 
@@ -752,10 +851,16 @@ static uint32_t fuse_setattr(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_attr_out *out = (struct fuse_attr_out *)
                                 (buf + sizeof(struct fuse_out_header));
-    DBG2("  n 0x%llx, m 0%o", ih->nodeid, in->mode);
-    DBG2("  uid %i, gid %i", in->uid, in->gid);
-    DBG2("  sz %llu, at %llu,", in->size, in->atime);
-    DBG1("  mt %llu", in->mtime);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_SETATTR:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, m 0%o",
+               ih->nodeid, in->mode);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  uid %i, gid %i",
+               in->uid, in->gid);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  sz %llu, at %llu,",
+               in->size, in->atime);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  mt %llu", in->mtime);
+    }
 
     oh->error = dav_setattr((dav_node *) ((size_t) ih->nodeid), ih->uid,
                             in->valid & FATTR_MODE, in->mode,
@@ -765,7 +870,6 @@ static uint32_t fuse_setattr(void) {
                             in->valid & FATTR_MTIME, in->mtime,
                             in->valid & FATTR_SIZE, in->size);
 
-    DBG_RET(oh->error);
     if (oh->error != 0) {
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
@@ -785,6 +889,8 @@ static uint32_t fuse_stat(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_statfs_out *out = (struct fuse_statfs_out *)
                                   (buf + sizeof(struct fuse_out_header));
+    if (debug)
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_STATFS:");
 
     dav_stat st = dav_statfs();
 
@@ -793,11 +899,8 @@ static uint32_t fuse_stat(void) {
     out->st.bavail = st.bavail;
     out->st.files = st.files;
     out->st.ffree = st.ffree;
-    if (st.bsize > buf_size) {
-        out->st.bsize = (buf_size - 4096) & ~1023;
-    } else {
-        out->st.bsize = st.bsize;
-    }
+    out->st.bsize = (buf_size - sizeof(struct fuse_in_header)
+                     - sizeof(struct fuse_write_in) - 4095) & ~4095;
     out->st.namelen = st.namelen;
     out->st.frsize = 0;
     out->st.padding = 0;
@@ -806,7 +909,6 @@ static uint32_t fuse_stat(void) {
         out->st.spare[i] = 0;
 
     oh->error = 0;
-    DBG_RET(oh->error);
     return sizeof(struct fuse_out_header) + sizeof(struct fuse_statfs_out);
 }
 
@@ -819,13 +921,18 @@ static uint32_t fuse_write(void) {
     struct fuse_out_header *oh = (struct fuse_out_header *) buf;
     struct fuse_write_out *out = (struct fuse_write_out *)
                                  (buf + sizeof(struct fuse_out_header));
-    DBG2("  n 0x%llx, fd %llu", ih->nodeid, in->fh);
-    DBG2("  pid %i, flags 0%o", ih->pid, in->write_flags);
-    DBG2("  size %u, off %llu", in->size, in->offset);
+    if (debug) {
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "FUSE_WRITE:");
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  n 0x%llx, fd %llu",
+               ih->nodeid, in->fh);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  pid %i, flags 0%o",
+               ih->pid, in->write_flags);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "  size %u, off %llu",
+               in->size, in->offset);
+    }
 
     if (in->size > (buf_size - sizeof(struct fuse_in_header)
                     - sizeof(struct fuse_write_in))) {
-        DBG_RET(EINVAL);
         oh->error = -EINVAL;
         return sizeof(struct fuse_out_header);
     }
@@ -836,7 +943,6 @@ static uint32_t fuse_write(void) {
                           + sizeof(struct fuse_write_in),
                           in->size, in->offset);
 
-    DBG_RET(oh->error);
     if (oh->error != 0) {
         oh->error *= -1;
         return sizeof(struct fuse_out_header);
