@@ -45,7 +45,6 @@
 
 #include <sys/mount.h>
 
-#include "dav_debug.h"
 #include "defaults.h"
 #include "cache.h"
 #include "kernel_interface.h"
@@ -107,7 +106,7 @@ int dav_init_kernel_interface(int *dev, dav_run_msgloop_fn *msg_loop,
     seteuid(0);
 
     if (*kernel_fs == NULL)
-        *kernel_fs = strdup("coda");
+        *kernel_fs = strdup("fuse");
     if (*kernel_fs == NULL)
         abort();
 
@@ -120,6 +119,8 @@ int dav_init_kernel_interface(int *dev, dav_run_msgloop_fn *msg_loop,
                           owner, group, mode) == 0) {
                 free(*kernel_fs);
                 *kernel_fs = strdup("fuse");
+                if (*kernel_fs == NULL)
+                    abort();
                 mounted = 1;
                 error(0, 0, _("fuse device opened successfully"));
             } else {
@@ -137,6 +138,8 @@ int dav_init_kernel_interface(int *dev, dav_run_msgloop_fn *msg_loop,
             if (init_coda(dev, msg_loop, mdata) == 0) {
                 free(*kernel_fs);
                 *kernel_fs = strdup("coda");
+                if (*kernel_fs == NULL)
+                    abort();
                 error(0, 0, _("coda device opened successfully"));
             } else {
                 exit(EXIT_FAILURE);
@@ -197,7 +200,6 @@ static int init_coda(int *dev, dav_run_msgloop_fn *msg_loop, void **mdata) {
 
     int version = 0;
     ioctl(*dev, CIOC_KERNEL_VERSION, &version);
-    DBG1("  CODA_KERNEL_VERSION %u.", version);
     if (version == 2) {
 #if SIZEOF_VOID_P == 8
         error(0, 0,
