@@ -71,24 +71,34 @@ typedef struct {
     int askauth;              /* User config file, sys conf f., command line */
     int displayname;          /* User config file, system config file */
     int locks;                /* User config file, sys conf f., command line */
+    char * lock_owner;        /* User config file, system config file */
+    time_t lock_timeout;      /* User config file, system config file */
+    time_t lock_refresh;      /* User config file, system config file */
     int expect100;            /* User config file, system config file */
+    int if_match_bug;         /* User config file, system config file */
+    int drop_weak_etags;    /* User config file, system config file */
+    int allow_cookie;         /* User config file, system config file */
+    int precheck;             /* User config file, system config file */
+    int ignore_dav_header;    /* User config file, system config file */
+    time_t connect_timeout;   /* User config file, system config file */
     time_t read_timeout;      /* User config file, system config file */
+    time_t retry;             /* User config file, system config file */
+    time_t max_retry;         /* User config file, system config file */
     char * s_charset;         /* User config file, system config file */
+    char * header;            /* User config file, system config file */
     /* Cache */
     char *sys_cache;          /* System config file */
     char *cache_dir;          /* User config file */
     char *backup_dir;         /* User config file, system config file */
     size_t cache_size;        /* User config file, system config file */
     size_t table_size;        /* User config file, system config file */
-    time_t idle_time;         /* User config file, system config file */
     time_t dir_refresh;       /* User config file, system config file */
     time_t file_refresh;      /* User config file, system config file */
+    int delay_upload;        /* User config file, system config file */
     int gui_optimize;         /* User config file, system config file */
-    time_t retry;             /* User config file, system config file */
-    time_t max_retry;         /* User config file, system config file */
-    time_t lock_timeout;      /* User config file, system config file */
-    time_t lock_refresh;      /* User config file, system config file */
-    char * lock_owner;        /* User config file, system config file */
+    /* Debugging */
+    int debug;                /* User config file, system config file */
+    int neon_debug;           /* User config file, system config file */
 } dav_args;
 
 
